<?php

namespace Drupal\Tests\index_now_commerce\Unit;

use Drupal\commerce_product\Entity\Product;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\index_now\Service\IndexNowInterface;
use Drupal\index_now_commerce\CommerceProductOperations;
use Drupal\Tests\UnitTestCase;
use Drupal\user\Entity\Role;
use Drupal\user\RoleStorageInterface;
use Prophecy\Argument;

/**
 * Tests the CommerceProductOperations class.
 *
 * @coversDefaultClass \Drupal\index_now_commerce\CommerceProductOperations
 *
 * @group index_now
 */
class CommerceProductOperationsTest extends UnitTestCase {

  const CONFIG_NAME = 'index_now.settings';

  const CONFIG_EXCLUDE_NAME = 'exclude_commerce_product_types';

  const PERMISSION = 'access content';

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->configFactory = $this->prophesize(ConfigFactoryInterface::class);
    $this->entityTypeManager = $this->prophesize(EntityTypeManagerInterface::class);
    $this->indexNow = $this->prophesize(IndexNowInterface::class);

    $this->roleStorage = $this->prophesize(RoleStorageInterface::class);
    $this->entityTypeManager
      ->getStorage('user_role')
      ->willReturn($this->roleStorage->reveal());

    $this->CommerceProductOperations = new CommerceProductOperations(
      $this->configFactory->reveal(),
      $this->entityTypeManager->reveal(),
      $this->indexNow->reveal()
    );
  }

  /**
   * @covers ::isIndexable
   */
  public function testProductIsUnpublished(): void {
    $product = $this->prophesize(Product::class);

    $product->isPublished()
      ->willReturn(FALSE);

    $this->roleStorage->load(Argument::any())
      ->ShouldNotBeCalled();

    $this->assertTrue(!$this->CommerceProductOperations->isIndexable($product->reveal()));
  }

  /**
   * @covers ::isIndexable
   */
  public function testUserCantAccessContent(): void {
    $product = $this->prophesize(Product::class);

    $product->isPublished()
      ->willReturn(TRUE);

    $role_prophecy = $this->prophesize(Role::class);
    $role_prophecy->hasPermission(self::PERMISSION)
      ->willReturn(FALSE);
    $this->roleStorage->load(Argument::any())
      ->ShouldBeCalled()
      ->willReturn($role_prophecy->reveal());

    $this->configFactory
      ->getEditable(self::CONFIG_NAME)
      ->ShouldNotBeCalled();

    $this->assertTrue(!$this->CommerceProductOperations->isIndexable($product->reveal()));
  }

  /**
   * @covers ::isIndexable
   */
  public function testProductTypeIsExcluded(): void {
    $product = $this->prophesize(Product::class);

    $product->isPublished()
      ->willReturn(TRUE);

    $role_prophecy = $this->prophesize(Role::class);
    $role_prophecy->hasPermission(self::PERMISSION)
      ->willReturn(TRUE);
    $this->roleStorage->load(Argument::any())
      ->ShouldBeCalled()
      ->willReturn($role_prophecy->reveal());

    $this->config = $this->prophesize(Config::class);
    $this->config
      ->get(self::CONFIG_EXCLUDE_NAME)
      ->willReturn(['foo']);
    $this->configFactory
      ->getEditable(self::CONFIG_NAME)
      ->ShouldBeCalled()
      ->willReturn($this->config->reveal());

    $product->bundle()
      ->ShouldBeCalled()
      ->willReturn('foo');

    $this->assertTrue(!$this->CommerceProductOperations->isIndexable($product->reveal()));
  }

  /**
   * @covers ::isIndexable
   */
  public function testProductIsIndexable(): void {
    $product = $this->prophesize(Product::class);

    $product->isPublished()
      ->willReturn(TRUE);

    $role_prophecy = $this->prophesize(Role::class);
    $role_prophecy->hasPermission(self::PERMISSION)
      ->willReturn(TRUE);
    $this->roleStorage->load(Argument::any())
      ->ShouldBeCalled()
      ->willReturn($role_prophecy->reveal());

    $this->config = $this->prophesize(Config::class);
    $this->config
      ->get(self::CONFIG_EXCLUDE_NAME)
      ->willReturn(['foo']);
    $this->configFactory
      ->getEditable(self::CONFIG_NAME)
      ->ShouldBeCalled()
      ->willReturn($this->config->reveal());

    $product->bundle()
      ->ShouldBeCalled()
      ->willReturn('bar');

    $this->assertTrue($this->CommerceProductOperations->isIndexable($product->reveal()));
  }

}
